"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "compatibility.js": "83d9c77748dafa9db49ae77d7f3d8fb0",
    "core.js": "5d08475f03adb5969bd31855d5ca0cfd",
    "frontend.html": "1533f44c55927e814294de757cd7eada",
    "mdi.html": "1cc8593d3684f7f6f3b3854403216f77",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-config.html": "39f00f769faa63ee61f1fe6fc85d67f7",
    "panels/ha-panel-dev-event.html": "2db9c218065ef0f61d8d08db8093cad2",
    "panels/ha-panel-dev-info.html": "61610e015a411cfc84edd2c4d489e71d",
    "panels/ha-panel-dev-service.html": "415552027cb083badeff5f16080410ed",
    "panels/ha-panel-dev-state.html": "d70314913b8923d750932367b1099750",
    "panels/ha-panel-dev-template.html": "567fbf86735e1b891e40c2f4060fec9b",
    "panels/ha-panel-hassio.html": "1d954cfe5f47c4be3cf4f6f5db9a83b2",
    "panels/ha-panel-history.html": "89062c48c76206cad1cec14ddbb1cbb1",
    "panels/ha-panel-iframe.html": "d920f0aa3c903680f2f8795e2255daab",
    "panels/ha-panel-logbook.html": "6dd6a16f52117318b202e60f98400163",
    "panels/ha-panel-map.html": "31c592c239636f91e07c7ac232a5ebc4",
    "panels/ha-panel-zwave.html": "a81f82b48439da80286798558f414a2e",
    "websocket_test.html": "575de64b431fe11c3785bf96d7813450"
}
